# -*- coding: utf-8 -*-
## -----------------------------------------------------------------
## Copyright (C)2013 ITYOU - www.ityou.de - support@ityou.de
## -----------------------------------------------------------------
"""
Only for testing
"""
from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from AccessControl import getSecurityManager
from AccessControl import Unauthorized
from Products.CMFCore.permissions import ModifyPortalContent, AddPortalContent
     
class CheckPermissionsView(BrowserView):
    """ Test Permissions
    """

    def __call__(self):

        context = aq_inner(self.context)
        request = context.REQUEST        
        sm =      getSecurityManager()

        print "------------------"
        print context.title_or_id()
        print sm.checkPermission(AddPortalContent, context)

        return         
    




# -*- coding: utf-8 -*-
import json

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

class AjaxSorting(BrowserView):
    """ Sorts the given documents 
    """
    def __call__(self):
        """ When called it iterates over the documents and sets new sorting numbers
        Parameters: "order" = uid,uid,uid
        
        Returns: Ordering (list of ids)
        """
        context = aq_inner(self.context)
        reference_tool = getToolByName(self, 'reference_catalog')
        docs = context.REQUEST.get("order").split(",")
        
        ordering = []
        
        for idx,doc in enumerate(docs):
            obj = reference_tool.lookupObject(doc)
            context.getOrdering().moveObjectToPosition(obj.getId(), idx)
        
        return json.dumps(context.getOrdering().idsInOrder())


from StringIO import StringIO
import logging
from plone.portlets.utils import unregisterPortletType
from zope.component.hooks import getSite
from zope.component import getSiteManager
from zope.component import getUtilitiesFor
from zope.component import adapts
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.component import getUtility
from zope.component import getMultiAdapter
from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletManagerRenderer
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletAssignmentSettings
from plone.portlets.interfaces import IPortletAssignment
from Products.CMFCore.utils import getToolByName

def remove_portlet_assignment(self):
    """Removes the portlet Assignment
    """
    content = getSite()
    for manager_name in [ "plone.leftcolumn", "plone.rightcolumn" ]:
        print "Checking: " + manager_name
        manager = getUtility(IPortletManager, name=manager_name, context=content)
        mapping = getMultiAdapter((content, manager), IPortletAssignmentMapping, context=content)
        for ap  in ['DragAndDrop']:
            if ap in mapping:
                logging.info( "\t Remove %s" % ap)
                del mapping[ap] 

    
def uninstall(self):
    """We need to uninstall the portlets GS???"""
    out = StringIO()
    logging.info("Uninstalling ITYOU DragDrop" )
    remove_portlet_assignment(self)
    print >> out, "Successfully uninstalled ityou.dragdrop"

    return out.getvalue()

    



# -*- coding: utf-8 -*-
## -----------------------------------------------------------------
## Copyright (C)2013 ITYOU - www.ityou.de - support@ityou.de
## -----------------------------------------------------------------
"""
This module contains the sqlite database interface
"""
import os
import logging

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, Unicode
from sqlalchemy.orm import sessionmaker

# --- psql -----------------------------------------------------
from ityou.esi.theme import PSQL_URI
# --- /psql -----------------------------------------------------

BASE = declarative_base()

class DragDrop(BASE):
    """ DragDrop Database
    user_id: User Id
    uid: UID of the Document
    """
    __tablename__ = 'dragdrop'
    
    id              = Column(Integer, primary_key=True)
    user_id         = Column(Unicode)
    uid             = Column(Unicode)

class DBApi(object):
    """ DB Util
    """
    def __init__(self):
        """Initialize Database
        """
        ## --- psql ----------------------------------------------------------
        engine  = create_engine(PSQL_URI,  client_encoding='utf8', echo=False)
        ## --- /psql ---------------------------------------------------------

        self.session = sessionmaker(bind=engine)

        BASE.metadata.create_all(engine)
    
    def addObject(self, user_id, uid):
        """ Adds the object to the user's clipboard
        """
        res = ""
        try:
            se = self.session()
            q = se.query(DragDrop).filter(DragDrop.user_id == user_id, DragDrop.uid == uid)
            if not q.first():
                d = DragDrop(
                           user_id = user_id,
                           uid = uid
                           )
                se.add(d)
                se.commit()
                res = True
        except:
            logging.error('Error while excecuting dragdrop addObject')            
        finally:
            se.close()
        return res

    
    def removeObject(self, user_id, uid):
        """ Removes the object from the user's clipboard
        """

        try:
            se = self.session()
            q = se.query(DragDrop).filter(DragDrop.user_id == user_id, DragDrop.uid == uid)
            q.delete()
            se.commit()

        except:
            logging.error('Error while excecuting dragdrop removeObject') 
        finally:
            se.close()
        
        return True

        
    def getObjects(self, user_id):
        """ Get Objects for user
        """
        uids = []
        try:
            se = self.session()
            q = se.query(DragDrop.uid).filter(DragDrop.user_id == user_id)            
            for uid in q.all():
                uids.append(uid[0])
        except:
            logging.error('Error while excecuting dragdrop getObject')
        finally:
            se.close()

        return uids
        



# -*- coding: utf-8 -*-
## -----------------------------------------------------------------
## Copyright (C)2013 ITYOU - www.ityou.de - support@ityou.de
## -----------------------------------------------------------------
"""
This module contains all views of ityou.dragdrop
"""
from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.app.uuid.utils import uuidToObject

from AccessControl import getSecurityManager
from Products.CMFCore.permissions import AddPortalContent, DeleteObjects

from ..dbapi import DBApi
from .. import isProductAvailable
from .. import _
from .. import TRASH_FOLDER
from plone import api
import json
from exceptions import KeyError

DB = DBApi()
     
class AjaxDragDropView(BrowserView):
    """Ajax View which handles DragDrop events
    """

    def __call__(self):
        """ Checks the action given in the request and calls the 
        resulting function
        Returns JSON
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        
        action = request.get('action')
        mt = getToolByName(context, "portal_membership")
        if mt.isAnonymousUser():
            return False
        
        if not action:
            res = None
        elif action == 'add_object':
            res = self.addObject()
        elif action == 'remove_object':
            res = self.removeObject()
        elif action == 'get_objects':
            res = self.getObjects()
        elif action == 'copy_object':
            res = self.copyOrMoveObject(copy = True)
        elif action == 'move_object':
            res = self.copyOrMoveObject(move = True)
        elif action == 'move_to_trash':
            res = self.moveObjectToTrash()
        elif action == 'can_copy':
            res = self.canCopyOrMove()
        elif action == 'can_move':
            res = self.canCopyOrMove(move = True)
        elif action == 'can_delete':
            res = self.canCopyOrMove(delete = True)
        else:
            res = False
        
        return self.jsonResponse(context, res)

    
    def addObject(self):
        """ Checks the authenticated user id.
        Then calls the DBApi addObject function with the checked user id and
        uid.
        Returns the value returned by DBApi function as jSON (True / False).
        """
        context =     aq_inner(self.context)
        request =     context.REQUEST
        mt =          getToolByName(context, 'portal_membership')
        auth_member = mt.getAuthenticatedMember()

        if api.user.is_anonymous():
            return False

        user_id = auth_member.getId()
        uid = request.get('uid')
        return self.jsonResponse(context, DB.addObject(user_id, uid))


    def removeObject(self):
        """ Checks the authenticated user id.
        Then calls the DBApi addObject function with the checked user id and
        uid.
        Returns the value returned by DBApi function as jSON (True / False).
        """
        context =   aq_inner(self.context)
        request =   context.REQUEST
        mt =        getToolByName(context, 'portal_membership')

        auth_member = mt.getAuthenticatedMember()
        if api.user.is_anonymous():
            return False

        user_id = auth_member.getId()
        uid = request.get('uid')
        return DB.removeObject(user_id, uid)

    def getObjects(self):
        """ Get objects for the current user
        Returns: 
        [
            {
                uid,
                id,
                title,
                url,
                portal_type,
                thumbnail_url,
            }
            ,
            ...
        ]
        """
        context =   aq_inner(self.context)
        mt =        getToolByName(context, 'portal_membership')
        rt =        getToolByName(self, 'reference_catalog')

        auth_member = mt.getAuthenticatedMember()

        if api.user.is_anonymous():
            return False

        user_id = auth_member.getId()
        uids = DB.getObjects(user_id)

        objects = []
        for uid in uids:
            obj = uuidToObject(uid)
            if obj:
                object = {}
                object["uid"] = uid
                object["id"] = obj.getId()
                object["title"] = obj.title_or_id()
                object["description"] = obj.Description()
                object["portal_type"] = obj.portal_type
                object["url"] = obj.absolute_url()
                author_id = obj.Creator()
                author = mt.getMemberById(author_id)
                object["author"] = author.getProperty("fullname") or author_id
                object["created"] = context.toLocalizedTime(obj.created())
                object["edited"] = context.toLocalizedTime(obj.modification_date)
                object["icon"] = obj.getIcon()

                if isProductAvailable("ityou.thumbnails"):
                    from ityou.thumbnails.thumbnail import ThumbnailManager
                    object["thumbnail_url"] = ThumbnailManager().getThumbnail(obj)
                else:
                    object["thumbnail_url"] = ""
                objects.append(object)
            else:
                DB.removeObject(user_id, uid)
            
        return objects


    def copyOrMoveObject(self, move = False, copy = False):
        """ Copy object with given uid to current folder
        Returns UID of new Object or False
        """
        context =   aq_inner(self.context)
        request =   context.REQUEST
        rt =        getToolByName(self, 'reference_catalog')
        catalog =   getToolByName(context, 'portal_catalog')

        uid = request.get("uid")
        obj = rt.lookupObject(uid)
        creation_date = obj.created()

        if move:
            api.content.move(obj, context)
        elif copy:
            api.content.copy(obj, context)
        else:
            return False
        
        folder_path = '/'.join(context.getPhysicalPath())
        brains = catalog(path={'query': folder_path, 'depth': 1}, created=creation_date, sort_on='modified')
        
        new_uid = uid
        if copy:
            i = 1
            while new_uid == uid:
                new_obj = brains[-i].getObject()
                new_uid = new_obj.UID()
                i += 1

        return new_uid


    def moveObjectToTrash(self):
        """ Move object with given uid to trash folder
        """
        context =   aq_inner(self.context)
        request =   context.REQUEST
        mt =        getToolByName(context, 'portal_membership')
        rt =        getToolByName(self, 'reference_catalog')

        uid = request.get("uid")
        auth_member = mt.getAuthenticatedMember()
        home_folder = auth_member.getHomeFolder()

        if not self.canCopyOrMove(move=True, container=home_folder):
            return False
        if not home_folder:
            return False

        trash_folder = self._personal_trash_folder()
        if not trash_folder:
            return False

        obj = rt.lookupObject(uid)
        api.content.move(obj, trash_folder)
        return obj.UID()

    
    def canCopyOrMove(self, move = False, container = False, delete = False, context = None):
        """ Checks if current user is allowed to copy file with given uid into current folder
        Returns True / False
        """
        context = context or aq_inner(self.context)
        request = context.REQUEST
        mt =      getToolByName(context, 'portal_membership')
        sm =      getSecurityManager()
        rt =      getToolByName(self, 'reference_catalog')

        if delete:
            container   = self._personal_trash_folder()
        elif not container:
            container = context

        if sm.checkPermission(AddPortalContent, container):
            if move or delete:
                uid = request.get("uid")
                obj = rt.lookupObject(uid)
                if not obj:
                    parent = context
                else:
                    parent = obj.aq_parent
                if not sm.checkPermission(DeleteObjects, parent):
                    return False
            return True
        else:
            return False


    def _personal_trash_folder(self):
        """returns trash folder object in the home folder of the user
        If it doesn't exist - create one
        """
        context     = aq_inner(self.context)

        mt          = getToolByName(context, 'portal_membership')
        if not mt.getAuthenticatedMember():
            return False

        home_folder = mt.getAuthenticatedMember().getHomeFolder()
        if not home_folder:
            return False

        try:
            trash_folder= home_folder[TRASH_FOLDER]
        except KeyError:
            try:
                trash_folder = api.content.create(container= home_folder, type="Folder", id=TRASH_FOLDER, title=context.translate("Trash"))
                trash_folder.setLayout("thumbnail_listing")
            except:
                return False
        return trash_folder        

        
    def jsonResponse(self, context, data):
        """ Returns Json Data in Callback function
        """
        request =  context.REQUEST
        callback = request.get('callback','')        
        request.response.setHeader("Content-type","application/json")
        if callback:
            cb = callback + "(%s);"
            return cb % json.dumps(data)
        else:
            return json.dumps(data)




# -*- coding: utf-8 -*-
from Acquisition import aq_inner

from plone.app.portlets.portlets import base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.portlets.interfaces import IPortletDataProvider
from zope.formlib import form
from zope.interface import implements

from .. import _

class IDragDropPortlet(IPortletDataProvider):
    """
    """

class Assignment(base.Assignment):
    implements(IDragDropPortlet)
    
    title = _(u"Drag & Drop")

class Renderer(base.Renderer):
    
    _template = ViewPageTemplateFile('dragdrop.pt')
    
    def render(self):

        context = aq_inner(self.context)
        mt = getToolByName(context, 'portal_membership')
        if mt.isAnonymousUser():
            return
        try:
            self.trash_folder = mt.getAuthenticatedMember().getHomeFolder()["trash"].absolute_url()
        except:
            self.trash_folder = "#"
        if not mt.isAnonymousUser() and context.defaultView() == "thumbnail_listing":
            return self._template()
              
    @property
    def available(self):
        return True

class AddForm(base.AddForm):
    form_fields = form.Fields(IDragDropPortlet)
    label = _(u"Add Drag&Drop portlet")
    description = _(u"A portlet where you can copy or move documents by copy & paste.")

    def create(self, data):
        return Assignment()
    


# -*- coding: utf-8 -*-
## -----------------------------------------------------------------
## Copyright (C)2013 ITYOU - www.ityou.de
## -----------------------------------------------------------------
"""
This module initialize the product ityou.dragdrop
"""
from zope.component.hooks import getSite
from Globals import INSTANCE_HOME
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('ityou.dragdrop')


DB_LOCATION    = '/'.join(INSTANCE_HOME.split('/')[:-2]) + "/var/sqlite3"
DB_DRAGDROP    = "sqlite:///"+ DB_LOCATION +  "/ityou.follow.db"
TABLE_DRAGDROP = 'dragdrop'


# Name des Trash-Orders, der im Benutzerordner abgelegt wird
TRASH_FOLDER   =  "trash"

def isProductAvailable(product):
    """Check if a product is installed and return True, 
    else return False
    """
    qui = getSite().portal_url.getPortalObject().portal_quickinstaller
    installed_products = qui.listInstalledProducts()
    for prod in installed_products:
        if prod["id"] == product and prod["status"] == "installed":
            return True
    return False

def initialize(context):
    """Initializer called when used as a Zope 2 product."""
